@@ -203,18 +203,10 @@ computeTestStatistic <- function(object, partable=NULL, samplestats=NULL,
     # degrees of freedom
     df <- getDF(partable)
 
-    # handle constraints
+    # handle equality constraints (note: we ignore inequality constraints, 
+    # active or not!)
     if(nrow(object@con.jac) > 0L) {
-        # Mplus seems to ignore inequality constraints
-        if(options$mimic == "Mplus") {
-            df <- ( df + length(attr(object@con.jac, "ceq.idx")) )
-        } else {
-            df <- ( df + length(attr(object@con.jac, "ceq.idx")) 
-                       + length(attr(object@con.jac, "cin.idx"))
-                       # FIXME:
-                       # we should not count inactive constraints?
-                       - length(attr(object@con.jac, "inactive.idx")) )
-        }
+        df <- ( df + length(attr(object@con.jac, "ceq.idx")) )
     }
 
     if(test == "none") {
@@ -230,17 +222,33 @@ computeTestStatistic <- function(object, partable=NULL, samplestats=NULL,
     fx <- attr(x, "fx")
     fx.group <- attr(fx, "fx.group")
 
-    # always compute `standard' test statistic
-    ## FIXME: the NFAC is now implicit in the computation of fx...
-    NFAC <- 2 * unlist(samplestats@nobs)
-    if(options$estimator == "ML" && options$likelihood == "wishart") {
-        # first divide by two
-        NFAC <- NFAC / 2
-        NFAC <- NFAC - 1
-        NFAC <- NFAC * 2
-    }
+    if(estimator != "PML") {
+        # always compute `standard' test statistic
+        ## FIXME: the NFAC is now implicit in the computation of fx...
+        NFAC <- 2 * unlist(samplestats@nobs)
+        if(options$estimator == "ML" && options$likelihood == "wishart") {
+            # first divide by two
+            NFAC <- NFAC / 2
+            NFAC <- NFAC - 1
+            NFAC <- NFAC * 2
+        }
+        chisq.group <- fx.group * NFAC
+    } else {
+        # for estimator PML, we compute the loglikelihood for the 
+        # `unrestricted' model, and then compute the LRT (-2 logl - logl_un)
+        group.fx <- numeric( samplestats@ngroups )
+        for(g in 1:samplestats@ngroups) {
+            group.fx <- estimator.PML(Sigma.hat = samplestats@cov[[g]],
+                                      TH        = samplestats@th[[g]],
+                                      th.idx    = object@th.idx[[g]],
+                                      num.idx   = object@num.idx[[g]],
+                                      X         = data@X[[g]])
+        }
+        chisq.group <- 2 * (fx.group - group.fx) # LRT per group
 
-    chisq.group <- fx.group * NFAC
+        cat("model fx = \n"); print(fx.group)
+        cat("unres fx = \n"); print(group.fx)
+    }
 
     # check for negative values
     chisq.group[which(chisq.group < 0)] <- 0.0
